package com.example.demo.orders.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.model.OrderGrouped;
import com.example.demo.orders.repository.OrderRepository;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

// Бизнес-логика для сущности "Заказ" (Заказ, содержащий книги)
@Service
public class OrderService {
    // Хранилище элементов
    private final OrderRepository repository;

    // Бизнес-логика для заказчиков (пользователей)
    private final UserService userService;

    // Конструктор
    public OrderService(OrderRepository repository, UserService userService) {
        this.repository = repository;
        this.userService = userService;
    }

    // Получить все элементы или по заданному фильтру
    @Transactional(readOnly = true)
    public List<OrderEntity> getAll(Long userId, Long typeId) {
        userService.get(userId);
        if (typeId <= 0L) {
            return repository.findByUserId(userId);
        }
        else {
            return repository.findByUserIdAndTypeId(userId, typeId);
        }
    }

    // Получить все элементы или по заданному фильтру (с пагинацией)
    @Transactional(readOnly = true)
    public Page<OrderEntity> getAll(long userId, long typeId, int page, int size) {
        final PageRequest pageRequest = PageRequest.of(page, size);
        userService.get(userId);
        if (typeId <= 0L) {
            return repository.findByUserId(userId, pageRequest);
        }
        return repository.findByUserIdAndTypeId(userId, typeId, pageRequest);
    }

    // Получить элемент по идентификатору
    @Transactional(readOnly = true)
    public OrderEntity get(Long userId, Long id) {
        userService.get(userId);
        return repository.findOneByUserIdAndId(userId, id)
            .orElseThrow(() -> new NotFoundException(OrderEntity.class, id));
    }

    // Создать элемент
    @Transactional
    public OrderEntity create(Long userId, OrderEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        final UserEntity existsUser = userService.get(userId);
        entity.setUser(existsUser);
        return repository.save(entity);
    }

    // Создать список заказов для пользователя
    @Transactional
    public List<OrderEntity> createAll(long userId, List<OrderEntity> entities) {
        if (entities == null || entities.isEmpty()) {
            throw new IllegalArgumentException("Orders list is null or empty");
        }
        final UserEntity existsUser = userService.get(userId);
        entities.forEach(entity -> entity.setUser(existsUser));
        return StreamSupport.stream(repository.saveAll(entities).spliterator(), false).toList();
    }

    // Изменить элемент
    @Transactional
    public OrderEntity update(Long userId, Long id, OrderEntity entity) {
        userService.get(userId);
        final OrderEntity existsEntity = get(userId, id);
        existsEntity.setType(entity.getType());
        existsEntity.setPrice(entity.getPrice());
        existsEntity.setCount(entity.getCount());
        return repository.save(existsEntity);
    }

    // Удалить элемент
    @Transactional
    public OrderEntity delete(Long userId, Long id) {
        userService.get(userId);
        final OrderEntity existsEntity = get(userId, id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    // Удалить все элементы
    @Transactional
    public void deleteAll() {
        repository.deleteAll();
    }

    // Получить список заказов, сгруппированных по типу
    @Transactional(readOnly = true)
    public List<OrderGrouped> getTotal(long userId) {
        userService.get(userId);
        return repository.getOrdersTotalByType(userId);
    }
}
